@@ -5,7 +5,7 @@ App::Virtualenv::Module - Module management for Perl virtual environment
 
 =head1 VERSION
 
-version 1.07
+version 1.09
 
 =head1 SYNOPSIS
 
@@ -33,7 +33,7 @@ BEGIN
 {
 	require Exporter;
 	# set the version for version checking
-	our $VERSION     = '1.07';
+	our $VERSION     = '1.09';
 	# Inherit from Exporter to export functions and variables
 	our @ISA         = qw(Exporter);
 	# Functions and variables which are exported by default
@@ -43,76 +43,38 @@ BEGIN
 }
 
 
-my @perl5lib = split(":", defined $ENV{PERL5LIB}? $ENV{PERL5LIB}: "");
-my %sitelib;
-$sitelib{$perl5lib[0]} = "perl5lib" if $perl5lib[0];
-unless (keys %sitelib)
-{
-	$sitelib{$Config{sitelib}} = "sitelib";
-	$sitelib{$Config{sitearch}} = "sitearch";
-}
-my $installed;
-App::Virtualenv::Module::reloadInstalled();
+my $inst = App::Virtualenv::Module::reloadInst();
 my $cb = CPANPLUS::Backend->new;
 
 
-sub reloadInstalled
+sub reloadInst
 {
-	$installed = ExtUtils::Installed->new(inc_override => [sort keys %sitelib]);
-	return;
-}
-
-sub moduleFiles
-{
-	my ($moduleName) = @_;
-	return unless grep($_ eq $moduleName, $installed->modules());
-	my %files;
-	for my $path (sort keys %sitelib)
-	{
-		for my $file ($installed->files($moduleName, "all", $path))
-		{
-			$files{$file} = $sitelib{$path};
-		}
-	}
-	return keys %files;
-}
-
-sub moduleDirectories
-{
-	my ($moduleName) = @_;
-	return unless grep($_ eq $moduleName, $installed->modules());
-	my %directories;
-	for my $path (sort keys %sitelib)
-	{
-		for my $directory ($installed->directories($moduleName, "all", $path))
-		{
-			$directories{$directory} = $sitelib{$path};
-		}
-	}
-	return keys %directories;
+	my @perl5lib = split(":", defined $ENV{PERL5LIB}? $ENV{PERL5LIB}: "");
+	$inst = ExtUtils::Installed->new(inc_override => [($perl5lib[0]? ("$perl5lib[0]/$Config{version}/$Config{archname}", "$perl5lib[0]/$Config{version}", "$perl5lib[0]/$Config{archname}", "$perl5lib[0]"): ($Config{sitearch}, $Config{sitelib}))]);
+	return $inst;
 }
 
 sub isInstalled
 {
 	my ($moduleName) = @_;
-	reloadInstalled();
-	return 0 unless grep($_ eq $moduleName, $installed->modules());
-	for my $path (sort keys %sitelib)
-	{
-		return 1 if $installed->files($moduleName, "all", $path);
-	}
-	return 0;
+	reloadInst();
+	return grep($_ eq $moduleName, $inst->modules()) > 0;
+}
+
+sub inPerlLib
+{
+	my ($moduleName) = @_;
+	return grep(-e $_."/".($moduleName =~ s/\:\:/\//r).".pm", ($Config{extrasarch}, $Config{extraslib}, $Config{archlib}, $Config{privlib})) > 0;
 }
 
 sub list
 {
 	my %params = @_;
-	reloadInstalled();
-	my @modules = $installed->modules();
+	reloadInst();
+	my @modules = $inst->modules();
 	for my $moduleName (sort {lc($a) cmp lc($b)} @modules)
 	{
-		my @files = moduleFiles($moduleName);
-		next unless @files;
+		next if lc($moduleName) eq 'perl';
 		if ($params{1})
 		{
 			say $moduleName;
@@ -122,7 +84,7 @@ sub list
 		my $len = length($space)-length($moduleName);
 		my $spaces = substr($space, -$len);
 		$spaces = "" if $len <= 0;
-		my $version = $installed->version($moduleName);
+		my $version = $inst->version($moduleName);
 		$version = "0" if not $version;
 		say "$moduleName$spaces $version";
 	}
@@ -135,14 +97,31 @@ sub install
 	my $force = $params{force}? 1: 0;
 	my $test = $params{test}? 1: 0;
 	my $soft = $params{soft}? 1: 0;
+	my $verbose = $params{verbose}? 1: 0;
 	my $result = 1;
+	state @installing;
+	state @installInfos;
+	push @installing, "";
 	for my $moduleName (@{$params{modules}})
 	{
+		pop @installing;
+		push @installing, $moduleName;
+		my $installInfo = {name => $moduleName, "success" => "", "fail" => "", "depth" => scalar(@installing)-1};
+		push @installInfos, $installInfo;
 		cp_msg("Looking for module $moduleName to install", 1);
+
+		if (lc($moduleName) eq "perl" or $moduleName eq "Config")
+		{
+			cp_msg("Module $moduleName is in Perl", 1);
+			$installInfo->{"success"} = "in Perl";
+			next;
+		}
+
 		my $mod = $cb->module_tree($moduleName);
 		if (not $mod)
 		{
 			cp_error("Module $moduleName is not found", 1);
+			$installInfo->{"fail"} = "find";
 			$result = 0;
 			next;
 		}
@@ -150,63 +129,66 @@ sub install
 		if ($mod->package_is_perl_core())
 		{
 			cp_msg("Module $moduleName is in Perl core", 1);
+			$installInfo->{"success"} = "in Perl core";
 			next;
 		}
 
-		if (not $force and grep({ my $inc = $_; $inc =~ /^\Q$Config{privlib}\E|\Q$Config{archlib}\E/ and -e $inc."/".($moduleName =~ s/\:\:/\//r).".pm"; } @INC))
+		if (not $force and inPerlLib($moduleName))
 		{
 			cp_msg("Module $moduleName is in Perl library", 1);
+			$installInfo->{"success"} = "in Perl library";
 			next;
 		}
 
 		my $installed = isInstalled($moduleName);
+		$installInfo->{"alreadyInstalled"} = $installed;
 		if (not $force and $installed and $mod->is_uptodate())
 		{
 			cp_msg("Module $moduleName is up to date", 1);
+			$installInfo->{"success"} = "up to date";
 			next;
 		}
 
 		cp_msg("Fetching module $moduleName", 1);
-		unless ($mod->fetch(verbose => 1))
+		unless ($mod->fetch(verbose => $verbose))
 		{
 			cp_error("Failed to fetch module $moduleName", 1);
+			$installInfo->{"fail"} = "fetch";
 			$result = 0;
 			next;
 		}
-		#cp_msg("Succeed to fetch module $moduleName", 1);
+		cp_msg("Succeed to fetch module $moduleName", 1);
 
 		cp_msg("Extracting module $moduleName", 1);
-		unless ($mod->extract(verbose => 1))
+		unless ($mod->extract(verbose => $verbose))
 		{
 			cp_error("Failed to extract module $moduleName", 1);
+			$installInfo->{"fail"} = "extract";
 			$result = 0;
 			next;
 		}
-		#cp_msg("Succeed to extract module $moduleName", 1);
+		cp_msg("Succeed to extract module $moduleName", 1);
 
 		cp_msg("Preparing module $moduleName", 1);
-		unless ($mod->prepare(verbose => 1))
+		unless ($mod->prepare(verbose => $verbose))
 		{
 			cp_error("Failed to prepare module $moduleName", 1);
+			$installInfo->{"fail"} = "prepare";
 			$result = 0;
 			next;
 		}
-		#cp_msg("Succeed to prepare module $moduleName", 1);
+		cp_msg("Succeed to prepare module $moduleName", 1);
 
 		unless ($soft)
 		{
 			cp_msg("Looking for prerequisites of module $moduleName", 1);
-			state @install;
-			push @install, $moduleName;
 			my $res = 1;
 			for my $ps (@{$mod->{_status}->{prereqs}})
 			{
 				delete $ps->{'perl'};
-				delete $ps->{'Config'};
-				delete $ps->{'Errno'};
 				for my $p (keys %$ps)
 				{
-					next if (grep($_ eq $p, @install));
+					next if (grep($_ eq $p, @installing));
 					unless (install(modules => [$p], test => $test))
 					{
 						$res = 0;
@@ -215,10 +197,10 @@ sub install
 				}
 				last unless $res;
 			}
-			pop @install;
 			unless ($res)
 			{
 				cp_error("Failed to install prerequisites of module $moduleName", 1);
+				$installInfo->{"fail"} = "install prerequisites";
 				$result = 0;
 				next;
 			}
@@ -228,25 +210,75 @@ sub install
 		if ($test)
 		{
 			cp_msg("Testing module $moduleName", 1);
-			unless ($mod->test(verbose => 1))
+			unless ($mod->test(verbose => $verbose))
 			{
 				cp_error("Failed to test module $moduleName", 1);
+				$installInfo->{"fail"} = "test";
 				$result = 0;
 				next;
 			}
-			#cp_msg("Succeed to test module $moduleName", 1);
+			cp_msg("Succeed to test module $moduleName", 1);
 		}
 
 		cp_msg("Installing module $moduleName", 1);
 		my $willBeStatus =  (not $installed)? "installed": "upgraded";
-		unless ($mod->install(verbose => 1, force => 1, skiptest => 1))
+		unless ($mod->install(verbose => $verbose, force => 1, skiptest => 1))
 		{
 			cp_error("Module $moduleName could not be $willBeStatus", 1);
+			$installInfo->{"fail"} = "install";
 			$result = 0;
 			next;
 		}
 		cp_msg("Module $moduleName has been successfully $willBeStatus", 1);
+		$installInfo->{"success"} = "installed";
 	}
+	pop @installing;
+
+	unless (@installing)
+	{
+		say "\nSummary:";
+		my ($installedCount, $failedCount) = (0, 0);
+		my $lastDepth = 0;
+		for my $installInfo (@installInfos)
+		{
+			my $msg;
+			if ($installInfo->{"fail"})
+			{
+				$msg = "is failed to $installInfo->{'fail'}";
+				$failedCount++;
+			} elsif ($installInfo->{"success"})
+			{
+				next if $installInfo->{'success'} ne "installed";
+				$msg = "is $installInfo->{'success'}";
+				$installedCount++;
+			}
+			my $spaces;
+			if ($installInfo->{depth} < $lastDepth)
+			{
+				$spaces = "";
+				for (0..$installInfo->{depth})
+				{
+					next unless $_;
+					$spaces .= "|---";
+				}
+				$spaces .= "|---|";
+				say $spaces;
+			}
+			$spaces = "";
+			for (0..$installInfo->{depth})
+			{
+				next unless $_;
+				$spaces .= "|---";
+			}
+			$spaces .= "|-- ";
+			say $spaces.$installInfo->{name}." $msg";
+			$lastDepth = $installInfo->{depth};
+		}
+		say "|\n| Installed: $installedCount\n| Failed: $failedCount\n";
+
+		@installInfos = ();
+	}
+
 	return $result;
 }
 
@@ -254,14 +286,19 @@ sub remove
 {
 	my %params = @_;
 	my $force = $params{force}? 1: 0;
+	my $verbose = $params{verbose}? 1: 0;
 	my $result = 1;
+	state @removeInfos;
 	for my $moduleName (@{$params{modules}})
 	{
+		my $removeInfo = {name => $moduleName, "success" => "", "fail" => "", "depth" => 0};
+		push @removeInfos, $removeInfo;
 		cp_msg("Looking for module $moduleName to remove", 1);
 		my $mod = $cb->module_tree($moduleName);
 		if (not $mod)
 		{
 			cp_error("Module $moduleName is not found", 1);
+			$removeInfo->{"fail"} = "find";
 			$result = 0;
 			next;
 		}
@@ -270,23 +307,70 @@ sub remove
 		unless ($installed)
 		{
 			cp_msg("Module $moduleName is not installed", 1);
+			$removeInfo->{"success"} = "not installed";
 			next;
 		}
 
 		cp_msg("Removing module $moduleName", 1);
-		unless ($mod->uninstall(verbose => 1, force => $force, type => 'all'))
+		unless ($mod->uninstall(verbose => $verbose, force => $force, type => 'all'))
 		{
 			cp_error("Module $moduleName could not be removed", 1);
+			$removeInfo->{"fail"} = "remove";
 			$result = 0;
 			next;
 		}
 		cp_msg("Module $moduleName has been successfully removed", 1);
+		$removeInfo->{"success"} = "removed";
 	}
+
+	say "\nSummary:";
+	my ($removedCount, $failedCount) = (0, 0);
+	my $lastDepth = 0;
+	for my $removeInfo (@removeInfos)
+	{
+		my $msg;
+		if ($removeInfo->{"fail"})
+		{
+			$msg = "is failed to $removeInfo->{'fail'}";
+			$failedCount++;
+		} elsif ($removeInfo->{"success"})
+		{
+			next if $removeInfo->{'success'} ne "removed";
+			$msg = "is $removeInfo->{'success'}";
+			$removedCount++;
+		}
+		my $spaces;
+		if ($removeInfo->{depth} < $lastDepth)
+		{
+			$spaces = "";
+			for (0..$removeInfo->{depth})
+			{
+				next unless $_;
+				$spaces .= "|---";
+			}
+			$spaces .= "|---|";
+			say $spaces;
+		}
+		$spaces = "";
+		for (0..$removeInfo->{depth})
+		{
+			next unless $_;
+			$spaces .= "|---";
+		}
+		$spaces .= "|-- ";
+		say $spaces.$removeInfo->{name}." $msg";
+		$lastDepth = $removeInfo->{depth};
+	}
+	say "|\n| Removed: $removedCount\n| Failed: $failedCount\n";
+
+	@removeInfos = ();
+
 	return $result;
 }
 
 
 1;
+__END__
 =head1 AUTHOR
 
 Orkun Karaduman <orkunkaraduman@gmail.com>
